<?php

namespace Drupal\symfony_mailer_reroute\Plugin\EmailAdjuster;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\symfony_mailer\EmailInterface;
use Drupal\symfony_mailer\Plugin\EmailAdjuster\AddressAdjusterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a Symfony Mailer Email Adjuster for rerouting emails.
 *
 * @EmailAdjuster(
 *   id = "reroute_email",
 *   label = @Translation("Reroute email (if enabled)"),
 *   description = @Translation("Reroutes emails send from the site to a predefined email. Useful for test sites."),
 *   provider = "symfony_mailer",
 * )
 */
class RerouteEmail extends AddressAdjusterBase implements ContainerFactoryPluginInterface {

  /**
   * The name of the associated header.
   */
  protected const NAME = 'To';

  /**
   * The symfony_mailer_reroute.settings config object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->config = $config_factory->get('symfony_mailer_reroute.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(EmailInterface $email) {
    if ((bool) $this->config->get('enable') === TRUE) {
      parent::build($email);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form = parent::settingsForm($form, $form_state);
    $form['addresses']['#type'] = 'details';
    $form['addresses']['#title'] = $this->t('Reroute email to address(es)');
    $form['addresses']['#open'] = TRUE;
    return $form;
  }

}
